@@ -45,6 +45,7 @@
 #include "absl/strings/string_view.h"
 #include "src/core/call/metadata_batch.h"
 #include "src/core/channelz/channelz.h"
+#include "src/core/filter/blackboard.h"
 #include "src/core/lib/channel/channel_args.h"
 #include "src/core/lib/channel/channel_fwd.h"
 #include "src/core/lib/channel/channel_stack.h"
@@ -66,6 +67,7 @@
 #include "src/core/util/cpp_impl_of.h"
 #include "src/core/util/dual_ref_counted.h"
 #include "src/core/util/orphanable.h"
+#include "src/core/util/per_cpu.h"
 #include "src/core/util/random_early_detection.h"
 #include "src/core/util/ref_counted_ptr.h"
 #include "src/core/util/sync.h"
@@ -86,6 +88,9 @@ class ServerConfigFetcher
     virtual absl::StatusOr<grpc_core::ChannelArgs>
     UpdateChannelArgsForConnection(const grpc_core::ChannelArgs& args,
                                    grpc_endpoint* tcp) = 0;
+
+    virtual void UpdateBlackboard(const Blackboard* old_blackboard,
+                                  Blackboard* new_blackboard) = 0;
   };
 
   class WatcherInterface {
@@ -120,6 +125,7 @@ class ListenerStateTestPeer;
 
 class Server : public ServerInterface,
                public InternallyRefCounted<Server>,
+               public channelz::DataSource,
                public CppImplOf<Server, grpc_server> {
  public:
   // Filter vtable.
@@ -246,6 +252,10 @@ class Server : public ServerInterface,
     void RemoveLogicalConnection(
         ListenerInterface::LogicalConnection* connection);
 
+    grpc_error_handle SetupTransport(Transport* transport,
+                                     grpc_pollset* accepting_pollset,
+                                     const ChannelArgs& args);
+
     const MemoryQuotaRefPtr& memory_quota() const { return memory_quota_; }
 
     const ConnectionQuotaRefPtr& connection_quota() const {
@@ -282,6 +292,11 @@ class Server : public ServerInterface,
       grpc_core::Timestamp timestamp;
     };
 
+    struct BlackboardShard {
+      Mutex mu;
+      RefCountedPtr<Blackboard> blackboard ABSL_GUARDED_BY(&mu);
+    };
+
     void DrainConnectionsLocked() ABSL_EXCLUSIVE_LOCKS_REQUIRED(mu_);
 
     void OnDrainGraceTimer();
@@ -299,6 +314,7 @@ class Server : public ServerInterface,
     OrphanablePtr<ListenerInterface> listener_;
     grpc_closure destroy_done_;
     ConfigFetcherWatcher* config_fetcher_watcher_ = nullptr;
+    PerCpu<BlackboardShard> blackboards_;
     Mutex mu_;  // We could share this mutex with Listener implementations. It's
                 // a tradeoff between increased memory requirement and more
                 // granular critical regions.
@@ -318,6 +334,8 @@ class Server : public ServerInterface,
   explicit Server(const ChannelArgs& args);
   ~Server() override;
 
+  void AddData(channelz::DataSink sink) override;
+
   void Orphan() ABSL_LOCKS_EXCLUDED(mu_global_) override;
 
   const ChannelArgs& channel_args() const override { return channel_args_; }
@@ -355,7 +373,8 @@ class Server : public ServerInterface,
   // Takes ownership of a ref on resource_user from the caller.
   grpc_error_handle SetupTransport(Transport* transport,
                                    grpc_pollset* accepting_pollset,
-                                   const ChannelArgs& args)
+                                   const ChannelArgs& args,
+                                   const Blackboard* blackboard = nullptr)
       ABSL_LOCKS_EXCLUDED(mu_global_);
 
   void RegisterCompletionQueue(grpc_completion_queue* cq);
@@ -641,7 +660,7 @@ class Server : public ServerInterface,
                                             ClientMetadataHandle md);
   auto MatchAndPublishCall(CallHandler call_handler);
   absl::StatusOr<RefCountedPtr<UnstartedCallDestination>> MakeCallDestination(
-      const ChannelArgs& args);
+      const ChannelArgs& args, const Blackboard* blackboard);
 
   ChannelArgs const channel_args_;
   RefCountedPtr<channelz::ServerNode> channelz_node_;
