@@ -20,12 +20,12 @@
 #include <grpc/impl/channel_arg_names.h>
 #include <grpc/slice.h>
 #include <grpc/support/alloc.h>
-#include <grpc/support/port_platform.h>
 #include <grpc/support/string_util.h>
 
 #include <algorithm>
 #include <cstddef>
 #include <memory>
+#include <optional>
 #include <string>
 #include <utility>
 #include <vector>
@@ -35,12 +35,11 @@
 #include "absl/status/status.h"
 #include "absl/strings/str_cat.h"
 #include "absl/strings/string_view.h"
-#include "absl/types/optional.h"
 #include "envoy/service/status/v3/csds.upb.h"
+#include "src/core/config/core_configuration.h"
 #include "src/core/lib/channel/channel_args.h"
 #include "src/core/lib/debug/trace.h"
 #include "src/core/lib/event_engine/channel_args_endpoint_config.h"
-#include "src/core/lib/event_engine/default_event_engine.h"
 #include "src/core/lib/iomgr/error.h"
 #include "src/core/lib/iomgr/exec_ctx.h"
 #include "src/core/lib/slice/slice.h"
@@ -48,6 +47,7 @@
 #include "src/core/lib/transport/error_utils.h"
 #include "src/core/telemetry/metrics.h"
 #include "src/core/util/debug_location.h"
+#include "src/core/util/down_cast.h"
 #include "src/core/util/env.h"
 #include "src/core/util/load_file.h"
 #include "src/core/util/orphanable.h"
@@ -161,16 +161,16 @@ class GrpcXdsClient::MetricsReporter final : public XdsMetricsReporter {
                              absl::string_view resource_type,
                              uint64_t num_valid_resources,
                              uint64_t num_invalid_resources) override {
-    xds_client_.stats_plugin_group_.AddCounter(
+    xds_client_.stats_plugin_group_->AddCounter(
         kMetricResourceUpdatesValid, num_valid_resources,
         {xds_client_.key_, xds_server, resource_type}, {});
-    xds_client_.stats_plugin_group_.AddCounter(
+    xds_client_.stats_plugin_group_->AddCounter(
         kMetricResourceUpdatesInvalid, num_invalid_resources,
         {xds_client_.key_, xds_server, resource_type}, {});
   }
 
   void ReportServerFailure(absl::string_view xds_server) override {
-    xds_client_.stats_plugin_group_.AddCounter(
+    xds_client_.stats_plugin_group_->AddCounter(
         kMetricServerFailure, 1, {xds_client_.key_, xds_server}, {});
   }
 
@@ -192,8 +192,11 @@ const grpc_channel_args* g_channel_args ABSL_GUARDED_BY(*g_mu) = nullptr;
 NoDestruct<std::map<absl::string_view, GrpcXdsClient*>> g_xds_client_map
     ABSL_GUARDED_BY(*g_mu);
 char* g_fallback_bootstrap_config ABSL_GUARDED_BY(*g_mu) = nullptr;
+NoDestruct<std::shared_ptr<GrpcXdsBootstrap>> g_parsed_bootstrap
+    ABSL_GUARDED_BY(*g_mu);
 
-absl::StatusOr<std::string> GetBootstrapContents(const char* fallback_config) {
+absl::StatusOr<std::string> FindBootstrapContents()
+    ABSL_EXCLUSIVE_LOCKS_REQUIRED(*g_mu) {
   // First, try GRPC_XDS_BOOTSTRAP env var.
   auto path = GetEnv("GRPC_XDS_BOOTSTRAP");
   if (path.has_value()) {
@@ -214,10 +217,10 @@ absl::StatusOr<std::string> GetBootstrapContents(const char* fallback_config) {
     return std::move(*env_config);
   }
   // Finally, try fallback config.
-  if (fallback_config != nullptr) {
+  if (g_fallback_bootstrap_config != nullptr) {
     GRPC_TRACE_LOG(xds_client, INFO)
         << "Got bootstrap contents from fallback config";
-    return fallback_config;
+    return g_fallback_bootstrap_config;
   }
   // No bootstrap config found.
   return absl::FailedPreconditionError(
@@ -225,7 +228,23 @@ absl::StatusOr<std::string> GetBootstrapContents(const char* fallback_config) {
       "not defined");
 }
 
-GlobalStatsPluginRegistry::StatsPluginGroup
+absl::StatusOr<std::shared_ptr<GrpcXdsBootstrap>> GetOrCreateGlobalBootstrap()
+    ABSL_EXCLUSIVE_LOCKS_REQUIRED(*g_mu) {
+  if (*g_parsed_bootstrap == nullptr) {
+    // First, find bootstrap contents.
+    auto bootstrap_contents = FindBootstrapContents();
+    if (!bootstrap_contents.ok()) return bootstrap_contents.status();
+    GRPC_TRACE_LOG(xds_client, INFO)
+        << "xDS bootstrap contents: " << *bootstrap_contents;
+    // Parse bootstrap.
+    auto bootstrap = GrpcXdsBootstrap::Create(*bootstrap_contents);
+    if (!bootstrap.ok()) return bootstrap.status();
+    *g_parsed_bootstrap = std::move(*bootstrap);
+  }
+  return *g_parsed_bootstrap;
+}
+
+std::shared_ptr<GlobalStatsPluginRegistry::StatsPluginGroup>
 GetStatsPluginGroupForKeyAndChannelArgs(absl::string_view key,
                                         const ChannelArgs& channel_args) {
   if (key == GrpcXdsClient::kServerKey) {
@@ -245,12 +264,15 @@ GetStatsPluginGroupForKeyAndChannelArgs(absl::string_view key,
 }  // namespace
 
 absl::StatusOr<RefCountedPtr<GrpcXdsClient>> GrpcXdsClient::GetOrCreate(
-    absl::string_view key, const ChannelArgs& args, const char* reason) {
+    absl::string_view key, const ChannelArgs& args, const char* reason,
+    std::shared_ptr<GrpcXdsBootstrap> bootstrap_override) {
   // If getting bootstrap from channel args, create a local XdsClient
   // instance for the channel or server instead of using the global instance.
-  absl::optional<absl::string_view> bootstrap_config = args.GetString(
+  std::optional<absl::string_view> bootstrap_config = args.GetString(
       GRPC_ARG_TEST_ONLY_DO_NOT_USE_IN_PROD_XDS_BOOTSTRAP_CONFIG);
   if (bootstrap_config.has_value()) {
+    GRPC_TRACE_LOG(xds_client, INFO)
+        << "xDS bootstrap contents: " << *bootstrap_config;
     auto bootstrap = GrpcXdsBootstrap::Create(*bootstrap_config);
     if (!bootstrap.ok()) return bootstrap.status();
     grpc_channel_args* xds_channel_args = args.GetPointer<grpc_channel_args>(
@@ -261,7 +283,8 @@ absl::StatusOr<RefCountedPtr<GrpcXdsClient>> GrpcXdsClient::GetOrCreate(
         MakeRefCounted<GrpcXdsTransportFactory>(channel_args),
         GetStatsPluginGroupForKeyAndChannelArgs(key, args));
   }
-  // Otherwise, use the global instance.
+  // Otherwise, check the global map to see if the XdsClient instance
+  // for this key already exists.
   MutexLock lock(g_mu);
   auto it = g_xds_client_map->find(key);
   if (it != g_xds_client_map->end()) {
@@ -270,18 +293,16 @@ absl::StatusOr<RefCountedPtr<GrpcXdsClient>> GrpcXdsClient::GetOrCreate(
       return xds_client.TakeAsSubclass<GrpcXdsClient>();
     }
   }
-  // Find bootstrap contents.
-  auto bootstrap_contents = GetBootstrapContents(g_fallback_bootstrap_config);
-  if (!bootstrap_contents.ok()) return bootstrap_contents.status();
-  GRPC_TRACE_LOG(xds_client, INFO)
-      << "xDS bootstrap contents: " << *bootstrap_contents;
-  // Parse bootstrap.
-  auto bootstrap = GrpcXdsBootstrap::Create(*bootstrap_contents);
-  if (!bootstrap.ok()) return bootstrap.status();
-  // Instantiate XdsClient.
+  // The XdsClient doesn't exist, so we'll create it.
+  std::shared_ptr<GrpcXdsBootstrap> bootstrap = std::move(bootstrap_override);
+  if (bootstrap == nullptr) {
+    auto global_bootstrap = GetOrCreateGlobalBootstrap();
+    if (!global_bootstrap.ok()) return global_bootstrap.status();
+    bootstrap = std::move(*global_bootstrap);
+  }
   auto channel_args = ChannelArgs::FromC(g_channel_args);
   auto xds_client = MakeRefCounted<GrpcXdsClient>(
-      key, std::move(*bootstrap), channel_args,
+      key, std::move(bootstrap), channel_args,
       MakeRefCounted<GrpcXdsTransportFactory>(channel_args),
       GetStatsPluginGroupForKeyAndChannelArgs(key, args));
   g_xds_client_map->emplace(xds_client->key(), xds_client.get());
@@ -309,7 +330,8 @@ GrpcXdsClient::GrpcXdsClient(
     absl::string_view key, std::shared_ptr<GrpcXdsBootstrap> bootstrap,
     const ChannelArgs& args,
     RefCountedPtr<XdsTransportFactory> transport_factory,
-    GlobalStatsPluginRegistry::StatsPluginGroup stats_plugin_group)
+    std::shared_ptr<GlobalStatsPluginRegistry::StatsPluginGroup>
+        stats_plugin_group)
     : XdsClient(
           bootstrap, transport_factory,
           grpc_event_engine::experimental::GetDefaultEventEngine(),
@@ -321,10 +343,10 @@ GrpcXdsClient::GrpcXdsClient(
                        .value_or(Duration::Seconds(15)))),
       key_(key),
       certificate_provider_store_(MakeOrphanable<CertificateProviderStore>(
-          static_cast<const GrpcXdsBootstrap&>(this->bootstrap())
+          DownCast<const GrpcXdsBootstrap&>(this->bootstrap())
               .certificate_providers())),
       stats_plugin_group_(std::move(stats_plugin_group)),
-      registered_metric_callback_(stats_plugin_group_.RegisterCallback(
+      registered_metric_callback_(stats_plugin_group_->RegisterCallback(
           [this](CallbackMetricReporter& reporter) {
             ReportCallbackMetrics(reporter);
           },
@@ -342,6 +364,7 @@ void GrpcXdsClient::Orphaned() {
   auto it = g_xds_client_map->find(key_);
   if (it != g_xds_client_map->end() && it->second == this) {
     g_xds_client_map->erase(it);
+    if (g_xds_client_map->empty()) g_parsed_bootstrap->reset();
   }
 }
 
@@ -360,9 +383,9 @@ namespace {
 std::vector<RefCountedPtr<GrpcXdsClient>> GetAllXdsClients() {
   MutexLock lock(g_mu);
   std::vector<RefCountedPtr<GrpcXdsClient>> xds_clients;
-  for (const auto& key_client : *g_xds_client_map) {
+  for (const auto& [_, client] : *g_xds_client_map) {
     auto xds_client =
-        key_client.second->RefIfNonZero(DEBUG_LOCATION, "DumpAllClientConfigs");
+        client->RefIfNonZero(DEBUG_LOCATION, "DumpAllClientConfigs");
     if (xds_client != nullptr) {
       xds_clients.emplace_back(xds_client.TakeAsSubclass<GrpcXdsClient>());
     }
@@ -425,6 +448,7 @@ void SetXdsChannelArgsForTest(grpc_channel_args* args) {
 void UnsetGlobalXdsClientsForTest() {
   MutexLock lock(g_mu);
   g_xds_client_map->clear();
+  g_parsed_bootstrap->reset();
 }
 
 void SetXdsFallbackBootstrapConfig(const char* config) {
@@ -439,7 +463,6 @@ void SetXdsFallbackBootstrapConfig(const char* config) {
 
 // The returned bytes may contain NULL(0), so we can't use c-string.
 grpc_slice grpc_dump_xds_configs(void) {
-  grpc_core::ApplicationCallbackExecCtx callback_exec_ctx;
   grpc_core::ExecCtx exec_ctx;
   return grpc_core::GrpcXdsClient::DumpAllClientConfigs();
 }
